package model

import (
	"fmt"
	"kpt-pasture/store/kptstore"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
)

type EventEstrus struct {
	Id                int64                           `json:"id"`
	CowId             int64                           `json:"cowId"`
	Lact              int32                           `json:"lact"`
	ExposeEstrusType  pasturePb.ExposeEstrusType_Kind `json:"exposeEstrusType"`
	FilterHigh        int32                           `json:"filter_high"`
	EstrusDate        string                          `json:"estrusDate"`
	ActiveDate        string                          `json:"activeDate"`
	LastEstrusDate    string                          `json:"lastEstrusDate"`
	Level             pasturePb.EstrusLevel_Kind      `json:"level"`
	IsPeak            pasturePb.IsShow_Kind           `json:"isPeak"`
	PerTwentyFourHigh int32                           `json:"perTwentyFourHigh"`
	Result            pasturePb.EstrusResult_Kind     `json:"result"`
	Remarks           string                          `json:"remarks"`
	IsShow            pasturePb.IsShow_Kind           `json:"isShow"`
	OperationId       int64                           `json:"operationId"`
	OperationName     string                          `json:"operationName"`
	MessageId         int64                           `json:"messageId"`
	MessageName       string                          `json:"messageName"`
	CreatedAt         int64                           `json:"createdAt"`
	UpdatedAt         int64                           `json:"updatedAt"`
}

func (e *EventEstrus) TableName() string {
	return "event_estrus"
}

func NewEventEstrus(
	exposeEstrusType pasturePb.ExposeEstrusType_Kind,
	cow *Cow,
	estrusDate, remarks string,
) *EventEstrus {
	return &EventEstrus{
		CowId:            cow.Id,
		Lact:             cow.Lact,
		ExposeEstrusType: exposeEstrusType,
		EstrusDate:       estrusDate,
		Remarks:          remarks,
	}
}

type EstrusSlice []*EventEstrus

func (e EstrusSlice) ToPB(
	dB *kptstore.DB,
	getCowInfo func(dB *kptstore.DB, cowId int64) *Cow,
	getCowLastEvent func(DB *kptstore.DB, cowId int64, eventCategoryId pasturePb.EventCategory_Kind) *EventCowLog,
) []*pasturePb.EstrusItems {
	res := make([]*pasturePb.EstrusItems, len(e))
	for i, v := range e {
		cowInfo := getCowInfo(dB, v.CowId)
		lastEventLog := getCowLastEvent(dB, v.CowId, pasturePb.EventCategory_Breed)
		planDay, optimumMatingTime := "", ""
		lastBreedEvnetDetails := ""
		if lastEventLog != nil {
			lastBreedEvnetDetails = fmt.Sprintf("%s %s", lastEventLog.EventTypeName, lastEventLog.EventDescription)
		}
		res[i] = &pasturePb.EstrusItems{
			Id:                     int32(v.Id),
			CowId:                  int32(v.CowId),
			DayAge:                 cowInfo.DayAge,
			Lact:                   cowInfo.Lact,
			Level:                  v.Level,
			PenName:                cowInfo.PenName,
			Status:                 v.IsShow,
			CalvingAge:             int32(cowInfo.CalvingAge),
			PlanDay:                planDay,
			MatingTimes:            cowInfo.MatingTimes,
			OptimumMatingStartTime: optimumMatingTime,
			OptimumMatingEndTime:   optimumMatingTime,
			LastBreedEvnetDetails:  lastBreedEvnetDetails,
		}
	}
	return res
}
