package model

import (
	"math"
	"strconv"
	"strings"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
)

type NeckRingOriginal struct {
	Id                 int64                         `json:"id"`
	Uuid               string                        `json:"uuid"`
	FrameId            int32                         `json:"frameId"`
	Low                int32                         `json:"low"`
	High               int32                         `json:"high"`
	Rumina             int32                         `json:"rumina"`
	Active             int32                         `json:"active"`
	Intake             int32                         `json:"intake"`
	Inactive           int32                         `json:"inactive"`
	Other              int32                         `json:"other"`
	Voltage            int32                         `json:"voltage"`
	Upper              int32                         `json:"upper"`
	Version            int32                         `json:"version"`
	Sign               int32                         `json:"sign"`
	Remain             int32                         `json:"remain"`
	Feed               int32                         `json:"feed"`
	Imei               string                        `json:"imei" `
	Temp               int32                         `json:"temp"`
	Gasp               int32                         `json:"gasp"`
	Hours              int32                         `json:"hours"`
	ActiveDate         string                        `json:"activeDate"`
	ActiveDateType     pasturePb.ActiveTimeType_Kind `json:"ActiveDateTimeType"`
	IsShow             pasturePb.IsShow_Kind         `json:"isShow"`
	ReceiveNumber      string                        `json:"receiveNumber"`
	ShortReceiveNumber string                        `json:"shortReceiveNumber"`
	CreatedAt          int64                         `json:"createdAt"`
	UpdatedAt          int64                         `json:"updatedAt"`
}

func (n *NeckRingOriginal) TableName() string {
	return "neck_ring_original"
}

var (
	AvgHours = int32(6)
	JoinKey  = "/"
)

func (n *NeckRingOriginal) IsAvgHours() {
	if n.ActiveDateType == pasturePb.ActiveTimeType_Two_Hours {
		n.Remain *= AvgHours
		n.Inactive *= AvgHours
		n.Active *= AvgHours
		n.Intake *= AvgHours
		n.Other *= AvgHours
		n.Gasp *= AvgHours
		n.High *= AvgHours
		n.High = int32(math.Min(1800, float64(n.Other)))
	}
}

type NeckRingOriginalMerge struct {
	Rumina         int32
	Inactive       int32
	Active         int32
	Intake         int32
	Other          int32
	High           int32
	Gasp           int32
	ActiveDateType pasturePb.ActiveTimeType_Kind
}

func (n *NeckRingOriginalMerge) IsMageData(data *NeckRingOriginal) {
	n.Rumina += data.Rumina
	n.Inactive += data.Inactive
	n.Active += data.Active
	n.Intake += data.Intake
	n.Other += data.Other
	n.Gasp += data.Gasp
	n.High += data.High
}

func (n *NeckRingOriginalMerge) SumAvg() {
	n.Rumina = n.Rumina / AvgHours * AvgHours
	n.Inactive = n.Inactive / AvgHours * AvgHours
	n.Active = n.Active / AvgHours * AvgHours
	n.Intake = n.Intake / AvgHours * AvgHours
	n.Other = n.Other / AvgHours * AvgHours
	n.Gasp = n.Gasp / AvgHours * AvgHours
	n.High = n.High / AvgHours * AvgHours
}

type NeckRingOriginalMap map[string]*NeckRingOriginalMerge

func (n NeckRingOriginalMap) ForMatData(defaultWeeklyActive int32, getCowInfo func(string) *Cow) []*NeckActiveHabit {
	res := make([]*NeckActiveHabit, 0)
	for key, v := range n {
		keyStrList := strings.Split(key, JoinKey)
		if len(keyStrList) != 3 {
			continue
		}
		imei := keyStrList[0]
		activeDate := keyStrList[1]
		frameId := keyStrList[2]
		frameIdInt, _ := strconv.Atoi(frameId)
		cowInfo := getCowInfo(imei)
		res = append(res, NewNeckActiveHabit(defaultWeeklyActive, int32(frameIdInt), activeDate, imei, cowInfo, v))
	}
	return res
}
